#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# glydget - an OpenGL widget toolkit
# Copyright (c) 2009 - Nicolas P. Rougier
#
# This file is part of glydget.
#
# glydget is free software: you can  redistribute it and/or modify it under the
# terms of  the GNU General  Public License as  published by the  Free Software
# Foundation, either  version 3 of the  License, or (at your  option) any later
# version.
#
# glydget is  distributed in the hope that  it will be useful,  but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy  of the GNU General Public License along with
# glydget. If not, see <http://www.gnu.org/licenses/>.
# -----------------------------------------------------------------------------
import pyglet
pyglet.options['debug_gl'] = False
import pyglet.gl as gl
from color import Color


class CubeGroup(pyglet.graphics.Group):
    ''' Cube fill group '''
    def set_state(self):
        gl.glPolygonOffset (1.0, 1.0)
        gl.glEnable (gl.GL_POLYGON_OFFSET_FILL)
        gl.glColor4f(1,1,1,1)
        gl.glEnable (gl.GL_COLOR_MATERIAL)
        gl.glColorMaterial(gl.GL_FRONT_AND_BACK, gl.GL_AMBIENT_AND_DIFFUSE)

    def unset_state(self):
        gl.glDisable(gl.GL_COLOR_MATERIAL)


class CubeOutlineGroup(pyglet.graphics.Group):
    ''' Cube outline group '''

    def set_state(self):
        gl.glDisable (gl.GL_POLYGON_OFFSET_FILL)
        gl.glEnable (gl.GL_LINE_SMOOTH)
        gl.glDisable (gl.GL_LIGHTING)

    def unset_state(self):
        gl.glDisable (gl.GL_LINE_SMOOTH)



class Cube(object):
    ''' Cube '''

    def __init__(self, position=[0.0,0.0,0.0], size=[1.0,1.0,1.0], fg_color = 'black',
                 bg_color=(255,255,80,255), batch=None, group=None):
        self._size = size
        self._position = position
        self._batch = batch or pyglet.graphics.Batch()
        self._group = CubeGroup(group)
        self._outline_group = CubeOutlineGroup(self._group)
        self._filled = True
        self._fg_color = Color(fg_color)
        self._bg_color = Color(bg_color)
        self._quad_list = None
        self._line_list = None
        self._build()


    def _build(self, *args):
        ''' Build cube '''

        if self._quad_list:
            self._quad_list.delete()
            self._quad_list = None
        if self._line_list:
            self._line_list.delete()
            self._line_list = None
        sx = self._size[0]/2.0
        sy = self._size[1]/2.0
        sz = self._size[2]/2.0
        x,y,z = self._position
        faces = [[[-sx+x,-sy+y, sz+z], [ sx+x,-sy+y, sz+z], [ sx+x, sy+y, sz+z], [-sx+x, sy+y, sz+z]],
                 [[-sx+x,-sy+y,-sz+z], [-sx+x, sy+y,-sz+z], [ sx+x, sy+y,-sz+z], [ sx+x,-sy+y,-sz+z]],
                 [[-sx+x, sy+y,-sz+z], [-sx+x, sy+y, sz+z], [ sx+x, sy+y, sz+z], [ sx+x, sy+y,-sz+z]],
                 [[-sx+x,-sy+y,-sz+z], [ sx+x,-sy+y,-sz+z], [ sx+x,-sy+y, sz+z], [-sx+x,-sy+y, sz+z]],
                 [[ sx+x,-sy+y,-sz+z], [ sx+x, sy+y,-sz+z], [ sx+x, sy+y, sz+z], [ sx+x,-sy+y, sz+z]],
                 [[-sx+x,-sy+y,-sz+z], [-sx+x,-sy+y, sz+z], [-sx+x, sy+y, sz+z], [-sx+x, sy+y,-sz+z]]]
        normals = [[ 0, 0, 1],
                   [ 0, 0,-1],
                   [ 0, 1, 0],
                   [ 0,-1, 0],
                   [ 1, 0, 0],
                   [-1, 0, 0] ]
        color = self._bg_color.RGBA
        quad_vertices = []
        quad_colors = []
        for face in faces:
            for point in face:
                x,y,z = point
                quad_vertices += point
                quad_colors += color
        quad_normals = []
        for normal in normals:
            quad_normals += normal*4
        color = self._fg_color.RGBA
        lines_vertices = []
        lines_colors = []
        for face in faces:
            for i in range(len(face)):
                x,y,z = face[i]
                lines_vertices += face[i]
                lines_colors += color
                j = (i+1) % (len(face))
                lines_vertices += face[j]
                lines_colors += color
        if self._filled:
            self._quad_list = self._batch.add(6*4, gl.GL_QUADS, self._group,
                                              ('v3f', quad_vertices),
                                              ('n3f', quad_normals),
                                              ('c4B', quad_colors))
        self._line_list = self._batch.add(6*4*2, gl.GL_LINES,  self._outline_group,
                                          ('v3f', lines_vertices),
                                          ('c4B', lines_colors))



    def _get_width(self):
        return self._size[0]

    def _set_width(self, width):
        self._size[0] = width
        self._build()

    width = property(_get_width, _set_width,
                 doc = 'Width')



    def _get_height(self):
        return self._size[2]

    def _set_height(self, height):
        self._size[2] = height
        self._build()

    height = property(_get_height, _set_height,
                      doc = 'Height')



    def _get_depth(self):
        return self._size[1]

    def _set_depth(self, depth):
        self._size[1] = depth
        self._build()

    depth = property(_get_depth, _set_depth,
                      doc = 'Depth')



    def _get_x(self):
        return self._position[0]

    def _set_x(self, x):
        self._position[0] = x
        self._build()

    x = property(_get_x, _set_x,
                 doc = 'X coordinate')



    def _get_y(self):
        return self._position[1]

    def _set_y(self, y):
        self._position[1] = y
        self._build()

    y = property(_get_y, _set_y,
                 doc = 'Y coordinate')



    def _get_z(self):
        return self._position[2]

    def _set_z(self, z):
        self._position[2] = z
        self._build()

    z = property(_get_z, _set_z,
                 doc = 'Z coordinate')


    def _get_fg_color(self):
        return self._fg_color

    def _set_fg_color(self, color):
        self._fg_color = Color(color)
        self._build()

    fg_color = property(_get_fg_color, _set_fg_color,
                        doc = 'Outline color')


    def _get_bg_color(self):
        return self._bg_color
    
    def _set_bg_color(self, color):
        self._bg_color = Color(color)
        self._build()
    bg_color = property(_get_bg_color, _set_bg_color,
                        doc = 'Outline color')


    def _get_filled(self):
        return self._filled
    
    def _set_filled(self, filled):
        self._filled = filled
        self._build()
    filled = property(_get_filled, _set_filled,
                        doc = 'Indicate whether cube is filled')


    def _get_batch(self):
        return self._batch

    batch = property(_get_batch,
                     doc = 'Cube batch')

