#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# glydget - an OpenGL widget toolkit
# Copyright (c) 2009 - Nicolas P. Rougier
#
# This file is part of glydget.
#
# glydget is free software: you can  redistribute it and/or modify it under the
# terms of  the GNU General  Public License as  published by the  Free Software
# Foundation, either  version 3 of the  License, or (at your  option) any later
# version.
#
# glydget is  distributed in the hope that  it will be useful,  but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy  of the GNU General Public License along with
# glydget. If not, see <http://www.gnu.org/licenses/>.
# -----------------------------------------------------------------------------
import pyglet
pyglet.options['debug_gl'] = False
import pyglet.gl as gl
import trackball
from cube import Cube


class SceneGroup(pyglet.graphics.Group):
    ''' Scene group '''

    def __init__(self, parent=None):
        ''' Create scene group. '''
        pyglet.graphics.Group.__init__(self, parent)
        self.trackball = trackball.Trackball(45.0,30.0,1.75)
        
    def set_state(self):
        ''' Setup pojection matrix and polygon offset fill mode. '''
        gl.glEnable(gl.GL_LIGHTING)
        gl.glEnable(gl.GL_LIGHT0)
        def vec(*args):
            return (gl.GLfloat * len(args))(*args)
        gl.glLightfv(gl.GL_LIGHT0, gl.GL_AMBIENT, vec(.1, .1, .1, 1.))
        gl.glLightfv(gl.GL_LIGHT0, gl.GL_DIFFUSE, vec(1., 1., 1., 1.))
        gl.glLightfv(gl.GL_LIGHT0, gl.GL_POSITION, vec(0., 1., 2., 1.))

        gl.glMatrixMode(gl.GL_PROJECTION)
        gl.glPushMatrix()
        gl.glLoadIdentity()
        angle = 15+105*(2-self.trackball.zoom)
        gl.gluPerspective(angle, window.width / float(window.height), .1, 1000)
        gl.glMatrixMode(gl.GL_MODELVIEW)
        gl.glPushMatrix()
        gl.glLoadIdentity()
        gl.glTranslatef(0, 0, -3)
        gl.glMultMatrixf(self.trackball.matrix)
        gl.glEnable(gl.GL_DEPTH_TEST)
        gl.glBlendFunc (gl.GL_SRC_ALPHA, gl.GL_ONE_MINUS_SRC_ALPHA)
        gl.glEnable(gl.GL_BLEND)

    def unset_state(self):
        ''' Unset projection matrix '''
        gl.glDisable(gl.GL_DEPTH_TEST)
        gl.glMatrixMode(gl.GL_PROJECTION)
        gl.glPopMatrix()
        gl.glMatrixMode(gl.GL_MODELVIEW)
        gl.glPopMatrix()
        gl.glDisable(gl.GL_LIGHTING)



if __name__ == '__main__':
    import glydget
    pyglet.options['debug_gl'] = False
    import pyglet.gl as gl
    from functools import partial

    window = pyglet.window.Window(width=640,height=480, resizable=True,vsync=0)
    pyglet.clock.schedule(lambda dt: None)
    fps_display = pyglet.clock.ClockDisplay()
    scene = SceneGroup()
    cube = Cube(group=scene)


    def quit(*args):
        pyglet.app.exit()


    @window.event
    def on_mouse_drag(x, y, dx, dy, button, modifiers):
        ''' Computes new rotation according to mouse drag '''
        x  = (x*2.0 - window.width)/float(window.width)
        dx = 2*dx/float(window.width)
        y  = (y*2.0 - window.height)/float(window.height)
        dy = 2*dy/float(window.height)
        scene.trackball.drag_to(x,y,dx,dy)
        return pyglet.event.EVENT_HANDLED


    @window.event
    def on_mouse_scroll(x, y, dx, dy):
        ''' Computes new zoom according to mouse scroll '''
        scene.trackball.zoom_to(x,y,dx,dy)
        return pyglet.event.EVENT_HANDLED


    variable = partial(glydget.Variable, namespace=locals())


    trackball_gui = glydget.Group('Trackball', 
        [variable('scene.trackball.phi', label=u'φ (phi)',
                  vmin=-180, vmax=180, vstep=1),
         variable('scene.trackball.theta', label=u'θ (theta)',
                  vmin=-90,  vmax=90, vstep=1),
         variable('scene.trackball.zoom', label=u'ρ (zoom)',
                  vmin=1, vmax=2, vstep=0.01)])

    cube_gui = glydget.Group('Cube',
                             [variable('cube.filled', label=u'Fill', action=cube._build),
                              glydget.Group('Color',
                                            [variable('cube.bg_color.R', label=u'Red',
                                                      vmin=0, vmax=255, action=cube._build),
                                             variable('cube.bg_color.G', label=u'Green',
                                                      vmin=0, vmax=255, action=cube._build),
                                             variable('cube.bg_color.B', label=u'Blue',
                                 vmin=0, vmax=255, action=cube._build)]),
                              glydget.Group('Position',
                                            [variable('cube.x', label=u'X',
                                                      vstep=0.01, action=cube._build),
                                             variable('cube.y', label=u'Y',
                                                      vstep=0.01, action=cube._build),
                                             variable('cube.z', label=u'Z',
                                                      vstep=0.01, action=cube._build)]),
                              glydget.Group('Size',
                                            [variable('cube.width', label=u'Width',
                                                      vmin=0, vstep=0.01, action=cube._build),
                                             variable('cube.height', label=u'Height',
                                                      vmin=0, vstep=0.01, action=cube._build),
                                             variable('cube.depth', label=u'Depth',
                                                      vmin=0, vstep=0.01, action=cube._build)])])
    
    dialog = glydget.Window ("Controls",
                             [trackball_gui,
                              cube_gui,
                              glydget.Button(u'Quit', quit)])
    dialog.show()
    dialog.move(5, window.height-5)
    window.push_handlers(dialog)


    background = glydget.Rectangle(0,window.height, window.width, window.height,
                                   colors= [  0,  0,127,255,   0,  0,127,255,
                                              255,255,255,255, 255,255,255,255], filled=True)

    @window.event
    def on_resize(width, height):
        background.begin_update()
        background.move(0,height)
        background.resize(width,height)
        background.end_update()

    @window.event
    def on_draw():
        gl.glClearColor(1,1,1,1)
        window.clear()
        background.batch.draw()
        cube.batch.draw()
        dialog.batch.draw()
        fps_display.draw()
    pyglet.app.run()
